@@ -1,4 +1,4 @@
-/* @(#) $Id: actsync.c,v 1.23 1996/12/16 20:51:30 brister Exp $ */
+/* @(#) $Id: actsync.c,v 1.16 1998/05/22 07:56:57 kondou Exp $ */
 /* @(#) Under RCS control in /usr/local/news/src/inn/local/RCS/actsync.c,v */
 /*
  * actsync - sync or merge two active files
@@ -64,7 +64,13 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/wait.h>
+#include <sys/mman.h>
+#include <syslog.h>
+#ifdef HAVE_WAIT_H
+# include <wait.h>
+#else
+# include <sys/wait.h>
+#endif
 #include <ctype.h>
 #include <math.h>
 #include <signal.h>
@@ -115,6 +121,12 @@ HiLowWater(name, hi, low, oldhi, oldlow)
     struct stat	statbuf;	/* stat of the file in the directory */
     char	*p;
     char	*q;
+    FILE	*fi;
+    struct stat sb;
+    int		count;
+    char	(*mapped)[][OVERINDEXPACKSIZE];
+    char	packed[OVERINDEXPACKSIZE];
+    OVERINDEX	index;
 
     /*
      * firewall
@@ -164,97 +176,141 @@ HiLowWater(name, hi, low, oldhi, oldlow)
     }
     *q = '\0';
 
-    /*
-     * attempt to open the directory
-     */
-    dp = opendir(dir);
-    if (dp == NULL) {
-	/* cannot scan the directory, assume not found or non-accessable */
-	return FALSE;
-    }
-
-    /*
-     * scan the directory for all numeric non-leading zero entries
-     * looking for the lowest and highest values
-     */
-    found_art = FALSE;
-    while ((ep = readdir(dp)) != NULL) {
-
-	/*
-	 * ignore if it it does not begin with [1-9]
-	 */
-	p = ep->d_name;
-	if (!CTYPE(isdigit, *p) || *p == '0') {
-	    /* first char is not [1-9], ignore */
-	    continue;
-	}
-
-	/*
-	 * ignore if contains a non-digit elsewhere in the name
-	 */
-	for (++p; *p; ++p) {
-	    if (!CTYPE(isdigit, *p)) {
-		break;
+    if (innconf->storageapi) {
+	sprintf(q, "/%s.index", innconf->overviewname);
+	if ((fi = fopen(dir, "r")) == NULL) {
+	    /* cannot open overview.index, assume not found or non-accessable */
+	    return FALSE;
+	}           
+	if (innconf->overviewmmap) {
+	    if (fstat(fileno(fi), &sb) < 0) {
+		fclose(fi);
+		return FALSE;
 	    }
+	    count = sb.st_size / OVERINDEXPACKSIZE;
+	    if (count == 0) {
+		fclose(fi);
+		return FALSE;
+	    }
+	    if ((mapped = (char (*)[][OVERINDEXPACKSIZE])mmap((MMAP_PTR)0, count * OVERINDEXPACKSIZE,
+		PROT_READ, MAP__ARG, fileno(fi), 0)) == (char (*)[][OVERINDEXPACKSIZE])-1) {
+		fclose(fi);
+		return FALSE;
+	    }
+	    fclose(fi);
+	    /* assumes .overview.index is sorted */
+	    UnpackOverIndex((*mapped)[0], &index);
+	    if (index.artnum < lowmark)
+		lowmark = index.artnum;
+	    UnpackOverIndex((*mapped)[count-1], &index);
+	    if (index.artnum > himark)
+		himark = index.artnum;
+	    if ((munmap((MMAP_PTR)mapped, count * OVERINDEXPACKSIZE)) < 0)
+		return FALSE;
+	} else {
+	    while (fread(&packed, OVERINDEXPACKSIZE, 1, fi) == 1) {
+		UnpackOverIndex(packed, &index);
+		if (index.artnum < lowmark)
+		    lowmark = index.artnum;
+		if (index.artnum > himark)
+		    himark = index.artnum;
+	    }
+	    fclose(fi);
 	}
-	if (*p) {
-	    /* found a non-digit, ignore this entry */
-	    continue;
+    } else {
+	/*
+	 * attempt to open the directory
+	 */
+	dp = opendir(dir);
+	if (dp == NULL) {
+	    /* cannot scan the directory, assume not found or non-accessable */
+	    return FALSE;
 	}
 
 	/*
-	 * convert numeric file name to a digit
+	 * scan the directory for all numeric non-leading zero entries
+	 * looking for the lowest and highest values
 	 */
-	num = atol(ep->d_name);
-
-	/*
-	 * look for a new low water mark
-	 *
-	 * Ignore this entry if it is not a file.
-	 */
-	if (num < lowmark) {
+	found_art = FALSE;
+	while ((ep = readdir(dp)) != NULL) {
 
 	    /*
-	     * ignore if this entry is not a file
-	     *
-	     * If the numeric name is too long, assume we have a file
+	     * ignore if it it does not begin with [1-9]
 	     */
-	    file_len = strlen(ep->d_name);
-	    if (dir_len+1+file_len < BUFSIZ+SMBUF) {
+	    p = ep->d_name;
+	    if (!CTYPE(isdigit, *p) || *p == '0') {
+		/* first char is not [1-9], ignore */
+		continue;
+	    }
 
-		/* 
-		 * form the filename 
-		 */
-		strncpy(path, dir, dir_len);
-		path[dir_len] = '/';
-		strncpy(path+dir_len+1, ep->d_name, file_len);
-		path[dir_len+1+file_len] = '\0';
-
-		/* 
-		 * skip if fie is not found or if it is not a regular file
-		 */
-		if (stat(path, &statbuf) < 0 || !S_ISREG(statbuf.st_mode)) {
-		    continue;
+	    /*
+	     * ignore if contains a non-digit elsewhere in the name
+	     */
+	    for (++p; *p; ++p) {
+		if (!CTYPE(isdigit, *p)) {
+		    break;
 		}
 	    }
+	    if (*p) {
+		/* found a non-digit, ignore this entry */
+		continue;
+	    }
 
 	    /*
-	     * we have a new low water mark
+	     * convert numeric file name to a digit
 	     */
-	    lowmark = num;
-	}
+	    num = atol(ep->d_name);
 
-	/*
-	 * note that we saw a numeric non-learing 0 file
-	 */
-	found_art = TRUE;
+	    /*
+	     * look for a new low water mark
+	     *
+	     * Ignore this entry if it is not a file.
+	     */
+	    if (num < lowmark) {
 
-	/*
-	 * look for a new hi water mark
-	 */
-	if (num > himark) {
-	    himark = num;
+		/*
+		 * ignore if this entry is not a file
+		 *
+		 * If the numeric name is too long, assume we have a file
+		 */
+		file_len = strlen(ep->d_name);
+		if (dir_len+1+file_len < BUFSIZ+SMBUF) {
+
+		    /* 
+		     * form the filename 
+		     */
+		    strncpy(path, dir, dir_len);
+		    path[dir_len] = '/';
+		    strncpy(path+dir_len+1, ep->d_name, file_len);
+		    path[dir_len+1+file_len] = '\0';
+
+		    /* 
+		     * skip if fie is not found or if it is not a regular file
+		     */
+		    if (stat(path, &statbuf) < 0 || !S_ISREG(statbuf.st_mode)) {
+			continue;
+		    }
+		}
+
+		/*
+		 * we have a new low water mark
+		 */
+		lowmark = num;
+	    }
+
+	    /*
+	     * note that we saw a numeric non-learing 0 file
+	     */
+	    found_art = TRUE;
+
+	    /*
+	     * look for a new hi water mark
+	     */
+	    if (num > himark) {
+		himark = num;
+	    }
 	}
+	(void)closedir(dp);
     }
 
     /* 
@@ -383,12 +439,6 @@ struct eqgrp {
 
 #define MIN_UNCHG (double)96.0	/* min % of host1 lines unchanged allowed */
 
-/*
- * path to ctlinnd command
- */
-/* =()<#define CTLINND_PATH "@<_PATH_NEWSBIN>@/ctlinnd">()= */
-#define CTLINND_PATH "/usr/news/bin/ctlinnd"
-
 #define DEV_NULL "/dev/null"	/* path to the bit bucket */
 #define CTLINND_NAME "ctlinnd"	/* basename of ctlinnd command */
 #define CTLINND_TIME_OUT "-t30"	/* seconds to wait before timeout */
@@ -403,13 +453,13 @@ struct eqgrp {
 #define NEWGRP_NOCHG 1		/* new group dir found but no hi/low change */
 #define NEWGRP_CHG 2		/* new group dir found but no hi/low change */
 
-#if !defined(DONT_USE_UNION_WAIT)
-#if !defined(WEXITSTATUS)
-#define WEXITSTATUS(status) ((status).w_retcode)
-#endif
-#if !defined(WTERMSIG)
-#define WTERMSIG(status) ((status).w_termsig)
-#endif
+#if defined(HAVE_UNION_WAIT)
+# if !defined(WEXITSTATUS)
+#  define WEXITSTATUS(status) ((status).w_retcode)
+# endif
+# if !defined(WTERMSIG)
+#  define WTERMSIG(status) ((status).w_termsig)
+# endif
 #endif
 
 /* -b macros */
@@ -512,7 +562,19 @@ main(argc, argv)
     char *host1;		/* host to change */
     char *host2;		/* comparison host */
 
+    /* First thing, set up logging and our identity. */
+    openlog("actsync", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
+
+    /*read in default info from inn.conf*/
+    if(ReadInnConf() < 0) {
+fprintf(stderr, "Couldn't init inn.conf\n");
+        return(1);
+    }
     /* process args */
+    if(ReadInnConf() < 0) {
+fprintf(stderr, "Couldn't init inn.conf\n");
+        return(1);
+    }
     process_args(argc, argv, &host1, &host2);
 
     /* obtain the active files */
@@ -885,11 +947,11 @@ process_args(argc, argv, host1, host2)
 
     /* determine default host name if needed */
     if (*host1 == NULL || strcmp(*host1, "-") == 0) {
-	def_serv = GetConfigValue(_CONF_SERVER);
+	def_serv = innconf->server;
 	*host1 = def_serv;
     }
     if (*host2 == NULL || strcmp(*host2, "-") == 0) {
-	def_serv = GetConfigValue(_CONF_SERVER);
+	def_serv = innconf->server;
 	*host2 = def_serv;
     }
     if (*host1 == NULL || *host2 == NULL) {
@@ -1080,7 +1142,7 @@ get_active(host, hostid, len, grp, errs)
 	is_file = 1;
 
 	/* setup to read the local file quickly */
-	if ((qp = QIOopen(host, QIO_BUFFER)) == NULL) {
+	if ((qp = QIOopen(host)) == NULL) {
 	    (void) fprintf(stderr,
 		"%s: can't read active file, %s\n", program, strerror(errno));
 	    exit(NOT_REACHED);
@@ -1094,7 +1156,7 @@ get_active(host, hostid, len, grp, errs)
 
 	/* open a connection to the server */
 	buff[0] = '\0';
-	if (NNTPconnect(host, &FromServer, &ToServer, buff) < 0) {
+	if (NNTPconnect(host, NNTP_PORT, &FromServer, &ToServer, buff) < 0) {
 	    (void) fprintf(stderr, "can't connect to server, %s\n",
 		    buff[0] ? buff : strerror(errno));
 	    exit(NOT_REACHED);
@@ -1111,7 +1173,7 @@ get_active(host, hostid, len, grp, errs)
 	}
 
 	/* setup to read the retrieved data quickly */
-	if ((qp = QIOfdopen((int)fileno(active), QIO_BUFFER)) == NULL) {
+	if ((qp = QIOfdopen((int)fileno(active))) == NULL) {
 	    (void) fprintf(stderr,
 		"%s: can't read temp file, %s\n", program, strerror(errno));
 	    (void) fclose(FromServer);
@@ -1671,7 +1733,7 @@ get_ignore(filename, len)
     max = CHUNK;
 
     /* setup to read the ignore file data quickly */
-    if ((qp = QIOopen(filename, QIO_BUFFER)) == NULL) {
+    if ((qp = QIOopen(filename)) == NULL) {
 	(void) fprintf(stderr,
 	    "%s: can't read ignore file: %s\n",
 	    program, filename);
@@ -3119,7 +3181,7 @@ exec_cmd(mode, cmd, grp, type, who)
     char buf[BUFSIZ+1];		/* interactive buffer */
     int pid;			/* pid of child process */
     int io[2];			/* pair of pipe descriptors */
-#if defined(DONT_USE_UNION_WAIT)
+#if !defined(HAVE_UNION_WAIT)
     int status;			/* wait status */
 #else
     union wait status;		/* wait status */
@@ -3235,14 +3297,15 @@ exec_cmd(mode, cmd, grp, type, who)
 	}
 
 	/* exec the ctlinnd command */
+	p = cpcatpath(innconf->pathbin, _PATH_CTLINND);
 	if (type == NULL) {
-	    execl(CTLINND_PATH,
+	    execl(p,
 		  CTLINND_NAME, CTLINND_TIME_OUT, cmd, grp, NULL);
 	} else if (who == NULL) {
-	    execl(CTLINND_PATH,
+	    execl(p,
 		  CTLINND_NAME, CTLINND_TIME_OUT, cmd, grp, type, NULL);
 	} else {
-	    execl(CTLINND_PATH,
+	    execl(p,
 		  CTLINND_NAME, CTLINND_TIME_OUT, cmd, grp, type, who, NULL);
 	}
 
