@@ -1,14 +1,14 @@
-/*  $Revision: 1.17 $
+/*  $Revision: 1.7 $
 **
 **  Read batchfiles on standard input and spew out batches.
 */
-#include "configdata.h"
 #include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include <fcntl.h>
 #include <ctype.h>
 #include <errno.h>
-#include <signal.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #if	defined(DO_NEED_TIME)
 #include <time.h>
@@ -16,8 +16,8 @@
 #include <sys/time.h>
 #include "paths.h"
 #include "libinn.h"
-#include "logging.h"
-#include "clibrary.h"
+#include "qio.h"
+#include <syslog.h> 
 #include "macros.h"
 
 
@@ -43,7 +43,7 @@ STATIC long	MaxArts;
 STATIC long	MaxBytes;
 STATIC SIGVAR	GotInterrupt;
 STATIC STRING	Separator = "#! rnews %ld";
-
+STATIC char	*ERRLOG;
 
 /*
 **  Start a batch process.
@@ -94,7 +94,6 @@ RequeueAndExit(Cookie, line, BytesInArt)
     static char	LINE1[] = "batcher %s times user %.3f system %.3f elapsed %.3f";
     static char	LINE2[] ="batcher %s stats batches %d articles %d bytes %ld";
     static char	NOWRITE[] = "batcher %s cant write spool %s\n";
-    static char	BATCHDIR[] = _PATH_BATCHDIR;
     char	temp[BUFSIZ];
     char	buff[BUFSIZ];
     int		i;
@@ -137,7 +136,7 @@ RequeueAndExit(Cookie, line, BytesInArt)
 
     /* Make an appropriate spool file. */
     if (Input == NULL)
-	(void)sprintf(temp, "%s/%s", BATCHDIR, Host);
+	(void)sprintf(temp, "%s/%s", innconf->pathoutgoing, Host);
     else
 	(void)sprintf(temp, "%s.bch", Input);
     if ((F = xfopena(temp)) == NULL) {
@@ -217,8 +216,6 @@ main(ac, av)
     char	*av[];
 {
     static char	SKIPPING[] = "batcher %s skipping \"%.40s...\" %s\n";
-    static char SPOOL[] = _PATH_SPOOL;
-    static char	BATCHDIR[] = _PATH_BATCHDIR;
     BOOL	Redirect;
     FILE	*F;
     STRING	AltSpool;
@@ -236,11 +233,15 @@ main(ac, av)
     int		ArtsInCB;
     int		length;
     struct stat	Sb;
+    BOOL Token;
+    QIOSTATE	*qp;
 
     /* Set defaults. */
+    if (ReadInnConf() < 0) exit(1);
     AltSpool = NULL;
     Redirect = TRUE;
     (void)umask(NEWSUMASK);
+    ERRLOG = COPY(cpcatpath(innconf->pathlog, _PATH_ERRLOG));
 
     /* Parse JCL. */
     while ((i = getopt(ac, av, "a:A:b:B:i:N:p:rs:S:v")) != EOF)
@@ -295,8 +296,9 @@ main(ac, av)
     Host = av[0];
     if ((Input = av[1]) != NULL) {
 	if (Input[0] != '/') {
-	    Input = NEW(char, STRLEN(BATCHDIR) + 1 + strlen(av[1]) + 1);
-	    (void)sprintf(Input, "%s/%s", BATCHDIR, av[1]);
+	    Input = NEW(char, strlen(innconf->pathoutgoing) +  1+
+					strlen(av[1]) + 1);
+	    (void)sprintf(Input, "%s/%s", innconf->pathoutgoing, av[1]);
 	}
 	if (freopen(Input, "r", stdin) == NULL) {
 	    (void)fprintf(stderr, "batcher %s cant open %s %s\n",
@@ -306,12 +308,12 @@ main(ac, av)
     }
 
     if (Redirect)
-	(void)freopen(_PATH_ERRLOG, "a", stderr);
+	(void)freopen(ERRLOG, "a", stderr);
 
     /* Go to where the articles are. */
-    if (chdir(SPOOL) < 0) {
+    if (chdir(innconf->patharticles) < 0) {
 	(void)fprintf(stderr, "batcher %s cant cd %s %s\n",
-		Host, SPOOL, strerror(errno));
+		Host, innconf->patharticles, strerror(errno));
 	exit(1);
     }
 
@@ -329,6 +331,7 @@ main(ac, av)
     (void)GetTimeInfo(&Now);
     STATbegin = TIMEINFOasDOUBLE(Now);
 
+    (void)SMinit();
     F = NULL;
     while (fgets(line, sizeof line, stdin) != NULL) {
 	/* Record line length in case we do an ftell. Not portable to
@@ -345,24 +348,31 @@ main(ac, av)
 	    continue;
 	if ((p = strchr(line, ' ')) != NULL) {
 	    *p++ = '\0';
-	    if ((BytesInArt = atol(p)) == 0) {
-		/* probably a "Wnm"-style (NNTP) batch file, work around it */
-		BytesInArt = -1;
-	    }
+	    /* Try to be forgiving of bad input. */
+	    BytesInArt = CTYPE(isdigit, *p) ? atol(p) : -1;
 	}
 	else
 	    BytesInArt = -1;
 
 	/* Strip of leading spool pathname. */
 	if (line[0] == '/'
-	 && line[STRLEN(SPOOL)] == '/'
-	 && EQn(line, SPOOL, STRLEN(SPOOL)))
-	    p = line + STRLEN(SPOOL) + 1;
+	 && line[strlen(innconf->patharticles)] == '/'
+	 && EQn(line, innconf->patharticles, strlen(innconf->patharticles)))
+	    p = line + strlen(innconf->patharticles) + 1;
 	else
 	    p = line;
 
 	/* Open the file. */
-	if ((artfd = open(p, O_RDONLY)) < 0) {
+	if (IsToken(p)) {
+	    if ((qp = QIOopen(p)) == NULL) {
+		if ((SMerrno != SMERR_NOENT) && (SMerrno != SMERR_UNINIT))
+		    (void)fprintf(stderr, SKIPPING,
+			    Host, p, SMerrorstr);
+		continue;
+	    }
+	    BytesInArt = -1;
+	    Token = TRUE;
+	} else if ((artfd = open(p, O_RDONLY)) < 0) {
 	    if (errno != ENOENT)
 		(void)fprintf(stderr, SKIPPING, Host, p, strerror(errno));
 	    if (AltSpool == NULL)
@@ -374,21 +384,28 @@ main(ac, av)
 			    Host, buff, strerror(errno));
 		continue;
 	    }
+	    Token = FALSE;
 	}
 
 	/* If we need to, get its size. */
-	if (BytesInArt < 0) {
-	    if (fstat(artfd, &Sb) < 0) {
-		(void)fprintf(stderr, SKIPPING, Host, line, strerror(errno));
-		(void)close(artfd);
-		continue;
+	if (BytesInArt <= 0) {
+	    if (Token) {
+		BytesInArt = 0;
+		while(QIOread(qp) != NULL)
+		    BytesInArt += QIOlength(qp) + 1;
+	    } else {
+		if (fstat(artfd, &Sb) < 0) {
+		    (void)fprintf(stderr, SKIPPING, Host, line, strerror(errno));
+		    (void)close(artfd);
+		    continue;
+		}
+		if (!S_ISREG(Sb.st_mode)) {
+		    (void)fprintf(stderr, SKIPPING, Host, line, "not a file");
+		    (void)close(artfd);
+		    continue;
+		}
+		BytesInArt = Sb.st_size;
 	    }
-	    if (!S_ISREG(Sb.st_mode)) {
-		(void)fprintf(stderr, SKIPPING, Host, line, "not a file");
-		(void)close(artfd);
-		continue;
-	    }
-	    BytesInArt = Sb.st_size;
 	}
 
 	/* Have an open article, do we need to open a batch?  This code
@@ -397,13 +414,19 @@ main(ac, av)
 	 * a bit more clear. */
 	if (F == NULL) {
 	    if (GotInterrupt) {
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		RequeueAndExit(Cookie, (char *)NULL, 0L);
 	    }
 	    if ((F = BATCHstart()) == NULL) {
 		(void)fprintf(stderr, "batcher %s cant startbatch %d %s\n",
 			Host, BATCHcount, strerror(errno));
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		break;
 	    }
 	    if (InitialString && *InitialString) {
@@ -426,7 +449,10 @@ main(ac, av)
 		else
 		    (void)fprintf(stderr, "batcher %s batch %d exit %d\n",
 			    Host, BATCHcount, BATCHstatus);
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		break;
 	    }
 	    ArtsInCB = 0;
@@ -436,19 +462,28 @@ main(ac, av)
 	    if ((MaxBatches > 0 && BATCHcount >= MaxBatches)
 	     || (MaxBytes > 0 && BytesWritten + BytesInArt >= MaxBytes)
 	     || (MaxArts > 0 && ArtsWritten + 1 >= MaxArts)) {
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		break;
 	    }
 
 	    if (GotInterrupt) {
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		RequeueAndExit(Cookie, line, BytesInArt);
 	    }
 
 	    if ((F = BATCHstart()) == NULL) {
 		(void)fprintf(stderr, "batcher %s cant startbatch %d %s\n",
 			Host, BATCHcount, strerror(errno));
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		break;
 	    }
 	    Cookie = ftell(stdin) - length;
@@ -463,7 +498,10 @@ main(ac, av)
 	    if (fprintf(F, "%s\n", buff) == EOF || ferror(F)) {
 		(void)fprintf(stderr, "batcher %s cant write separator %s\n",
 		    Host, strerror(errno));
-		(void)close(artfd);
+		if (Token)
+		    QIOclose(qp);
+		else
+		    (void)close(artfd);
 		break;
 	    }
 	}
@@ -471,16 +509,29 @@ main(ac, av)
 	/* Write the article.  In case of interrupts, retry the read but
 	 * not the fwrite because we can't check that reliably and
 	 * portably. */
-	while ((i = read(artfd, (POINTER)data, datasize)) > 0 || errno == EINTR)
-	    if (fwrite((POINTER)data, (SIZE_T)1, (SIZE_T)i, F) != i)
-	        break;
-	if (ferror(F)) {
-	    (void)fprintf(stderr, "batcher %s cant write article %s\n",
+	if (Token) {
+	    (void)QIOrewind(qp);
+	    while((p = QIOread(qp)) != NULL) {
+		if ((fprintf(F, "%s\n", p) == EOF) || ferror(F))
+		    break;
+	    }
+	    if (ferror(F)) {
+		QIOclose(qp);
+		break;
+	    }
+	    QIOclose(qp);
+	} else {
+	    while ((i = read(artfd, (POINTER)data, datasize)) > 0 || errno == EINTR)
+		if (fwrite((POINTER)data, (SIZE_T)1, (SIZE_T)i, F) != i)
+		    break;
+	    if (ferror(F)) {
+	        (void)fprintf(stderr, "batcher %s cant write article %s\n",
 		    Host, strerror(errno));
+	        (void)close(artfd);
+	        break;
+	    }
 	    (void)close(artfd);
-	    break;
 	}
-	(void)close(artfd);
 
 	/* Update the counts. */
 	BytesInCB += BytesInArt;
