@@ -1,4 +1,4 @@
-/*  $Revision: 1.9 $
+/*  $Revision: 1.11 $
 **  Connect to a remote site, and get news from it to offer to our local
 **  server.  Read list on stdin, or get it via NEWNEWS command.  Writes
 **  list of articles still needed to stdout.
@@ -20,6 +20,7 @@
 #include "dbz.h"
 #include "nntp.h"
 #include "macros.h"
+#include <syslog.h>  
 
 
 /*
@@ -131,9 +132,9 @@ SITEconnect(host)
 
     /* Connect and identify ourselves. */
     if (host)
-	i = NNTPconnect(host, &From, &To, (char *)NULL);
+	i = NNTPconnect(host, NNTP_PORT, &From, &To, (char *)NULL);
     else {
-	host = GetConfigValue(_CONF_SERVER);
+	host = innconf->server;
 	i = NNTPlocalopen(&From, &To, (char *)NULL);
     }
     if (i < 0) {
@@ -178,18 +179,7 @@ STATIC BOOL
 HIShaveit(mesgid)
     char		*mesgid;
 {
-    register char	*p;
-    datum		key;
-    datum		value;
-    char		buff[NNTP_STRLEN];
-
-    (void)strcpy(buff, mesgid);
-    for (p = key.dptr = buff; *p; p++)
-	if (*p == HIS_FIELDSEP || *p == '\n')
-	    *p = HIS_BADCHAR;
-    key.dsize = p - key.dptr + 1;
-    value = dbzfetch(key);
-    return value.dptr != NULL ? TRUE : FALSE;
+    return dbzexists(HashMessageID(mesgid));
 }
 
 
@@ -224,16 +214,22 @@ main(ac, av)
     FILE	*F;
     BOOL	Offer;
     BOOL	Error;
-    BOOL	Verbose;
+    BOOL	Verbose = FALSE;
     char	*Update;
     char	*p;
 
+    /* First thing, set up logging and our identity. */
+    openlog("nntpget", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);           
+
     /* Set defaults. */
     distributions = NULL;
     Groups = NULL;
     Since = NULL;
     Offer = FALSE;
     Update = NULL;
+
+    if (ReadInnConf() < 0) exit(1);
+
     (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
@@ -267,7 +263,7 @@ main(ac, av)
 	    break;
 	case 'o':
 	    /* Open the history file. */
-	    if (dbminit(_PATH_HISTORY) < 0) {
+	    if (!dbzinit(cpcatpath(innconf->pathdb, _PATH_HISTORY))) {
 		(void)fprintf(stderr, "Can't open history, %s\n",
 		    strerror(errno));
 		exit(1);
@@ -332,8 +328,7 @@ main(ac, av)
 	}
 
 	/* Create a temporary file. */
-	p = getenv("TMPDIR");
-	(void)sprintf(temp, "%s/nntpgetXXXXXX", p ? p : _PATH_TMP);
+	(void)sprintf(temp, "%s/nntpgetXXXXXX", innconf->pathtmp);
 	(void)mktemp(temp);
 	if ((F = fopen(temp, "w+")) == NULL) {
 	    (void)fprintf(stderr, "Can't open \"%s\", %s\n",
@@ -412,9 +407,15 @@ main(ac, av)
 	    break;
 	}
 	if (atoi(buff) != NNTP_ARTICLE_FOLLOWS_VAL) {
-	    if (Offer)
-		(void)SITEwrite(Local, ".", 1);
-	    continue;
+          if (Offer) {
+              (void)SITEwrite(Local, ".", 1);
+              if (!SITEread(Local, buff)) {
+                  (void)fprintf(stderr, "No reply after \"%s\", %s\n",
+                                mesgid, strerror(errno));
+                  break;
+              }
+          }
+          continue;
 	}
 
 	if (Verbose)
