@@ -1,4 +1,4 @@
-/*  $Revision: 1.42 $
+/*  $Revision: 1.47 $
 **
 **  Rebuild the history database.
 **
@@ -120,9 +120,9 @@ static void ARTreadschema(BOOL Overview)
 		Datep = fp;
 	    if (caseEQn(buff, MESSAGEID, STRLEN(MESSAGEID)-1))
 		Msgidp = fp;
-	    if (caseEQn(buff, EXPIRES, STRLEN(EXPIRES))-1)
+	    if (caseEQn(buff, EXPIRES, STRLEN(EXPIRES)-1))
 		Expp = fp;
-	    if (caseEQn(buff, XREF, STRLEN(XREF))-1)
+	    if (caseEQn(buff, XREF, STRLEN(XREF)-1))
 		Xrefp = fp;
 	    fp++;
 	}
@@ -347,8 +347,10 @@ STATIC void Rebuild(long size, BOOL IgnoreOld, BOOL Overwrite)
 		((q = strchr(save + 1, '@')) != NULL)) {
 		*(++q) = '\0';
 		if (!IsToken(save)) {
-		    fprintf(stderr, "Invalid token %s for hash %s, skipping\n", q, p);
-		    continue;
+		    /* assumes traditional spool */
+		    ionevalue.offset = where;
+		    ivalue = (void *)&ionevalue;
+		    break;
 		}
 		if (innconf->extendeddbz) {
 		    iextvalue.offset[HISTOFFSET] = where;
@@ -796,7 +798,8 @@ DoMemArt(ARTHANDLE *art, BOOL Overview, BOOL Update, FILE *out, FILE *index, BOO
 		for (p++; *p == ' '; p++);
 	        q = p;
 	    }
-	    i = fprintf(index, "[%s] %s\n", hash, q);
+	    if(*q)
+		i = fprintf(index, "[%s] %s\n", hash, q);
 	    if (i == EOF || ferror(index)) {
 		(void)fprintf(stderr, "Can't write index line, %s\n", strerror(errno));
 		exit(1);
@@ -848,6 +851,7 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
     int			crossnum;
     time_t		Arrived;
     char		buff[SPOOLNAMEBUFF];
+    BOOL		HasHash;
 
     if ((qp = QIOopen(ACTIVE)) == NULL) {
 	(void)fprintf(stderr, "Can't open %s, %s\n", ACTIVE, strerror(errno));
@@ -886,13 +890,9 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 		(void)fprintf(stderr, IGNORING, line);
 		continue;
 	    }
-	    switch (fields[0][0]) {
-	    case '[':
-		if (strlen(fields[0]) != ((sizeof(HASH) * 2) + 2)) {
-		    fprintf(stderr, "Invalid length for hash %s, skipping\n", fields[0]);   
-		    break;
-		}
-		if (i == 2) {
+	    if (i == 2) {
+		switch (fields[0][0]) {
+		case '[':
 		    if (out != NULL) {
 			i = fprintf(out, "%s%c%s\n", fields[0], HIS_FIELDSEP, fields[1]);
 			if (i == EOF || ferror(out)) {
@@ -901,12 +901,32 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 			}
 		    }
 		    break;
-		} else if (!RemoveOld) {
-		    /* just make index */
-		    if (!IsToken(fields[2])) {
-			fprintf(stderr, "Invalid token %s, skipping\n", fields[2]);   
-			break;
+		case '<':
+		    if (out != NULL) {
+			i = fprintf(out, "[%s]%c%s\n", HashToText(HashMessageID(fields[0])), HIS_FIELDSEP, fields[1]);
+			if (i == EOF || ferror(out)) {
+			    (void)fprintf(stderr, "Can't write history line, %s\n", strerror(errno));     
+			    exit(1);
+			}
 		    }
+		    break;
+		default:
+		    fprintf(stderr, "Invalid message-id \"%s\" in history text\n", fields[0]);
+		    break;
+		}
+		continue;
+	    }
+	    if (fields[0][0] == '[') {
+		if (strlen(fields[0]) != ((sizeof(HASH) * 2) + 2)) {
+		    fprintf(stderr, "Invalid length for hash %s, skipping\n", fields[0]);   
+		    continue;
+		} else
+		    HasHash = TRUE;
+	    } else
+		HasHash = FALSE;
+	    if (IsToken(fields[2])) {
+		if (!RemoveOld) {
+		    /* just make index */
 		    if (out != NULL) {
 			i = fprintf(out, "%s%c%s%c%s\n", fields[0], HIS_FIELDSEP, fields[1], HIS_FIELDSEP, fields[2]);
 			if (i == EOF || ferror(out)) {
@@ -918,7 +938,7 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 			token = TextToToken(fields[2]);
 			if (token.index < OVER_NONE) {
 			    if ((p = OVERretrieve(&token, &linelen)) == (char *)NULL)
-				break;
+				continue;
 			    if (OVERmmap) {
 				if (!OVERline)
 				    OVERline = NEW(char, MAXOVERLINE);
@@ -931,13 +951,13 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 			    }
 			    if (index != (FILE *)NULL) {
 			        if ((Xref = strstr(OVERline, "\tXref:")) == NULL) {
-				    break;
+				    continue;
 			        }
 			        if ((Xref = strchr(Xref, ' ')) == NULL)
-				    break;
+				    continue;
 			        for (Xref++; *Xref == ' '; Xref++);
 			        if ((Xref = strchr(Xref, ' ')) == NULL)
-				    break;
+				    continue;
 			        for (Xref++; *Xref == ' '; Xref++);
 			        if (!Xrefbuf)
 				    Xrefbuf = NEW(char, MAXOVERLINE);
@@ -964,16 +984,12 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 			    }
 			}
 		    }
-		    break;
-		}
-		if (!IsToken(fields[2])) {
-		    fprintf(stderr, "Invalid token %s, skipping\n", fields[2]);   
-		    break;
+		    continue;
 		}
 		token = TextToToken(fields[2]);
 		if ((art = SMretrieve(token, RETR_ALL)) == (ARTHANDLE *)NULL) {
 		    /* fprintf(stderr, "Cannot retrieve %s, skipping\n", fields[2]); */
-		    break;
+		    continue;
 		}
 		if (artbuff.Left == 0) {
 		    artbuff.Data = NEW(char, art->len);
@@ -993,22 +1009,11 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 		token = SMstore(arth);
 		if (token.type == TOKEN_EMPTY) {
 		    fprintf(stderr, "Cannot store %s, skipping\n", fields[0]);
-		    break;
+		    continue;
 		}
 		arth.token = &token;
 		DoMemArt(&arth, Overview, FALSE, out, index, TRUE);
-		break;
-	    case '<':
-		if (i == 2) {
-		    if (out != NULL) {
-			i = fprintf(out, "[%s]%c%s\n", HashToText(HashMessageID(fields[0])), HIS_FIELDSEP, fields[1]);
-			if (i == EOF || ferror(out)) {
-			    (void)fprintf(stderr, "Can't write history line, %s\n", strerror(errno));     
-			    exit(1);
-			}
-		    }
-		    break;
-		}
+	    } else {
 		if ((p = strchr(fields[1], HIS_SUBFIELDSEP)) == (char *)NULL)
 		    Arrived = atol(fields[1]);
 		else {
@@ -1022,13 +1027,16 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 		if (!ReadInMem(arts[0], &arth, Tradspooldir)) {
 		    /* maybe article is cancelled, just recored the hash */
 		    if (out != NULL) {
-			i = fprintf(out, "[%s]%c%s\n", HashToText(HashMessageID(fields[0])), HIS_FIELDSEP, fields[1]);
+			if (HasHash)
+			    i = fprintf(out, "%s%c%s\n", fields[0], HIS_FIELDSEP, fields[1]);
+			else
+			    i = fprintf(out, "[%s]%c%s\n", HashToText(HashMessageID(fields[0])), HIS_FIELDSEP, fields[1]);
 			if (i == EOF || ferror(out)) {
 			    (void)fprintf(stderr, "Can't write history line, %s\n", strerror(errno));     
 			    exit(1);
 			}
 		    }
-		    break;
+		    continue;
 		}
 		if (RemoveOld)
 		    i = 0;
@@ -1046,14 +1054,10 @@ TranslateFromHistory(FILE *out, char *OldHistory, char *Tradspooldir, BOOL Unlin
 		token = SMstore(arth);
 		if (token.type == TOKEN_EMPTY) {
 		    fprintf(stderr, "Cannot store %s, skipping\n", fields[0]);
-		    break;
+		    continue;
 		}
 		arth.token = &token;
 		DoMemArt(&arth, Overview, FALSE, out, index, TRUE);
-		break;
-	    default:
-		fprintf(stderr, "Invalid message-id \"%s\" in history text\n", fields[0]);
-		break;
 	    }
 	} else
 	    break;
@@ -1075,6 +1079,7 @@ DoArticle(QIOSTATE *qp, struct stat *Sbp, char *name, FILE *out, BOOL RemoveBad,
     time_t		Expires;
     time_t		Posted;
     int			i;
+    HASH		key;
 
     /* Read the file for Message-ID and Expires header. */
     Arrived = Sbp->st_mtime;
@@ -1142,28 +1147,45 @@ DoArticle(QIOSTATE *qp, struct stat *Sbp, char *name, FILE *out, BOOL RemoveBad,
 	return;
     }
 
+    key = HashMessageID(MessageID);
     if (Update) {
 	/* Server already know about this one? */
-	if (dbzexists(HashMessageID(MessageID)))
+	if (dbzexists(key))
 	    return;
     }
 
     /* Output the line. */
     if (Posted == 0)
 	Posted = Arrived;
-    if (Expires > 0)
-	i = fprintf(out, "%s%c%lu%c%lu%c%lu%c%s\n",
-                    MessageID, HIS_FIELDSEP,
+    if (innconf->storemsgid) {
+	if (Expires > 0)
+	    i = fprintf(out, "%s%c%lu%c%lu%c%lu%c%s\n",
+		    MessageID, HIS_FIELDSEP,
                     (unsigned long)Arrived, HIS_SUBFIELDSEP,
                     (unsigned long)Expires,
 		    HIS_SUBFIELDSEP, (unsigned long)Posted, HIS_FIELDSEP,
                     name);
-    else
-	i = fprintf(out, "%s%c%lu%c%s%c%lu%c%s\n",
-                    MessageID, HIS_FIELDSEP,
+	else
+	    i = fprintf(out, "%s%c%lu%c%s%c%lu%c%s\n",
+		    MessageID, HIS_FIELDSEP,
                     (unsigned long)Arrived, HIS_SUBFIELDSEP, HIS_NOEXP,
 		    HIS_SUBFIELDSEP, (unsigned long)Posted, HIS_FIELDSEP,
                     name);
+    } else {
+	if (Expires > 0)
+	    i = fprintf(out, "[%s]%c%lu%c%lu%c%lu%c%s\n",
+		    HashToText(key), HIS_FIELDSEP,
+                    (unsigned long)Arrived, HIS_SUBFIELDSEP,
+                    (unsigned long)Expires,
+		    HIS_SUBFIELDSEP, (unsigned long)Posted, HIS_FIELDSEP,
+                    name);
+	else
+	    i = fprintf(out, "[%s]%c%lu%c%s%c%lu%c%s\n",
+		    HashToText(key), HIS_FIELDSEP,
+                    (unsigned long)Arrived, HIS_SUBFIELDSEP, HIS_NOEXP,
+		    HIS_SUBFIELDSEP, (unsigned long)Posted, HIS_FIELDSEP,
+                    name);
+    }
     if (i == EOF || ferror(out)) {
 	(void)fprintf(stderr, "Can't write history line, %s\n",
 		strerror(errno));
