@@ -1,4 +1,4 @@
-/*  $Revision: 1.40 $
+/*  $Revision: 1.12 $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -21,6 +21,7 @@
 #include "paths.h"
 #include "libinn.h"
 #include "macros.h"
+#include <syslog.h>  
 
 
 #define FLUSH_ERROR(F)		(fflush((F)) == EOF || ferror((F)))
@@ -45,10 +46,10 @@ typedef struct _HEADER {
 } HEADER;
 
 
+STATIC char     *tmpPtr ;
 STATIC BOOL	Dump;
 STATIC BOOL	Revoked;
 STATIC BOOL	Spooling;
-STATIC char	SPOOLNEWS[] = _PATH_SPOOLNEWS;
 STATIC char	**OtherHeaders;
 STATIC char	NGSEPS[] = NG_SEPARATOR;
 STATIC char	SIGSEP[] = SIG_SEPARATOR;
@@ -536,9 +537,7 @@ FormatUserName(pwp, node)
 /*
 **  Check the Distribution header, and exit on error.
 */
-STATIC void
-CheckDistribution(p)
-    register char	*p;
+STATIC void CheckDistribution(char *p)
 {
     static char		SEPS[] = " \t,";
     register STRING	*dp;
@@ -593,7 +592,7 @@ ProcessHeaders(AddOrg, linecount, pwp)
     }
 
     /* Set From or Sender. */
-    if ((p = GetConfigValue(_CONF_FROMHOST)) == NULL)
+    if ((p = innconf->fromhost) == NULL)
 	PerrorExit(TRUE, "Can't get host name");
     if (HDR(_from) == NULL)
 	HDR(_from) = FormatUserName(pwp, p);
@@ -673,7 +672,7 @@ ProcessHeaders(AddOrg, linecount, pwp)
     if (HDR(_path) == NULL) {
 	i = strlen(Exclusions) + STRLEN(PATHFLUFF);
 #if	defined(DO_INEWS_PATH)
-	if ((p = GetFileConfigValue(_CONF_PATHHOST)) != NULL) {
+	if ((p = innconf->pathhost) != NULL) {
 	    i += strlen(p) + 1;
 	    HDR(_path) = NEW(char, i + 1);
 	    if (*p)
@@ -681,7 +680,7 @@ ProcessHeaders(AddOrg, linecount, pwp)
 	    else
 		(void)sprintf(HDR(_path), "%s%s", Exclusions, PATHFLUFF);
 	}
-	else if (GetFileConfigValue(_CONF_SERVER) != NULL) {
+	else if (innconf->server != NULL) {
 	    if ((p = GetFQDN()) == NULL) {
 		(void)fprintf(stderr, "Can't get host name, %s\n",
 			strerror(errno));
@@ -725,7 +724,7 @@ ProcessHeaders(AddOrg, linecount, pwp)
     /* Set Organization. */
     if (AddOrg
      && HDR(_organization) == NULL
-     && (p = GetConfigValue(_CONF_ORGANIZATION)) != NULL) {
+     && (p = innconf->organization) != NULL) {
 	HDR(_organization) = COPY(p);
     }
 
@@ -826,7 +825,6 @@ AppendSignature(UseMalloc, article, homedir, linesp)
 }
 
 
-#if	defined(DO_CHECK_INCLUDED_TEXT)
 /*
 **  See if the user has more included text than new text.  Simple-minded, but
 **  reasonably effective for catching neophyte's mistakes.  A line starting
@@ -864,7 +862,6 @@ CheckIncludedText(p, lines)
 	QuitServer(1);
     }
 }
-#endif	/* defined(DO_CHECK_INCLUDED_TEXT) */
 
 
 
@@ -898,8 +895,10 @@ MailArticle(group, article)
     (void)printf("posted, but mailed to the moderator for approval.\n");
 
     /* Now build up the command (ignore format/argument mismatch errors,
-     * in case %s isn't in _PATH_SENDMAIL) and send the headers. */
-    (void)sprintf(buff, _PATH_SENDMAIL, address);
+     * in case %s isn't in innconf->mta) and send the headers. */
+    if (innconf->mta == NULL)
+	PerrorExit(TRUE, "Can't start mailer - not set");
+    (void)sprintf(buff, innconf->mta, address);
     if ((F = popen(buff, "w")) == NULL)
 	PerrorExit(TRUE, "Can't start mailer");
     (void)fprintf(F, "To: %s\n", address);
@@ -997,7 +996,8 @@ ValidNewsgroups(hdr, F, article)
 				  group);
 		else {
 		    mailed = MailArticle(group, article);
-		    DISPOSE(DDend(h));
+                    tmpPtr = DDend(h) ;
+		    DISPOSE(tmpPtr);
 		    QuitServer(mailed ? 0 : 1);
 		}
 	    break;
@@ -1005,7 +1005,8 @@ ValidNewsgroups(hdr, F, article)
 	case NF_FLAG_NOLOCAL:
 	    (void)fprintf(stderr, "Postings to \"%s\" are not allowed here.\n",
 		    group);
-	    DISPOSE(DDend(h));
+            tmpPtr = DDend(h) ;
+	    DISPOSE(tmpPtr);
 	    return FALSE;
 	case NF_FLAG_EXCLUDED:
 	    (void)fprintf(stderr, "Warning:  \"%s\" is rejected here.\n",
@@ -1018,7 +1019,8 @@ ValidNewsgroups(hdr, F, article)
 	    (void)fprintf(stderr,
 		    "The newsgroup \"%s\" has been renamed to \"%s\".\n",
 		    group, p + 1);
-	    DISPOSE(DDend(h));
+            tmpPtr = DDend(h) ;
+	    DISPOSE(tmpPtr);
 	    return FALSE;
 	}
 
@@ -1026,7 +1028,8 @@ ValidNewsgroups(hdr, F, article)
 
     if (!FoundOne && !IsNewgroup) {
 	(void)fprintf(stderr, "No such newsgroups as \"%s\".\n", hdr);
-	DISPOSE(DDend(h));
+        tmpPtr = DDend(h) ;
+	DISPOSE(tmpPtr);
 	return FALSE;
     }
 
@@ -1100,7 +1103,6 @@ Spoolit(article, Length, deadfile)
     SIZE_T		Length;
     char		*deadfile;
 {
-    static char		SPOOLTEMP[] = _PATH_SPOOLTEMP;
     register HEADER	*hp;
     register FILE	*F;
     register int	i;
@@ -1109,9 +1111,9 @@ Spoolit(article, Length, deadfile)
     struct stat		Sb;
 
     /* Try to write to the spool dir, else the deadfile. */
-    if ((stat(SPOOLNEWS, &Sb) >= 0 && S_ISDIR(Sb.st_mode))
+    if ((stat(innconf->pathincoming, &Sb) >= 0 && S_ISDIR(Sb.st_mode))
      || (F = xfopena(deadfile)) == NULL) {
-	TempName(SPOOLTEMP, temp);
+	TempName(innconf->pathtmp, temp);
 	(void)umask(0);
 	if ((i = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) < 0
 	 || (F = fdopen(i, "w")) == NULL)
@@ -1138,7 +1140,7 @@ Spoolit(article, Length, deadfile)
 
     if (deadfile == NULL) {
 	/* Put the file in a good place. */
-	TempName(SPOOLNEWS, buff);
+	TempName(innconf->pathincoming, buff);
 	if (rename(temp, buff) < 0)
 	    PerrorExit(FALSE, "Can't rename spool file");
     }
@@ -1179,6 +1181,9 @@ main(ac, av)
     BOOL		AddOrg;
     SIZE_T		Length;
 
+    /* First thing, set up logging and our identity. */
+    openlog("inews", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);      
+
     /* Find out who we are. */
     if ((i = geteuid()) < 0)
 	PerrorExit(TRUE, "Can't get your user ID");
@@ -1190,6 +1195,9 @@ main(ac, av)
     Dump = FALSE;
     DoSignature = TRUE;
     AddOrg = TRUE;
+
+    if (ReadInnConf() < 0) exit(1);
+
     (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
@@ -1257,7 +1265,7 @@ main(ac, av)
     }
 
     /* Try to open a connection to the server. */
-    if (NNTPremoteopen(&FromServer, &ToServer, buff) < 0) {
+    if (NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff) < 0) {
 	Spooling = TRUE;
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
@@ -1295,23 +1303,20 @@ main(ac, av)
 	article = StripOffHeaders(article);
     for (i = 0, p = article; (p = strchr(p, '\n')) != NULL; i++, p++)
 	continue;
-#if	defined(DO_CHECK_INCLUDED_TEXT)
-    CheckIncludedText(article, i);
-#endif	/* defined(DO_CHECK_INCLUDED_TEXT) */
+    if (innconf->checkincludedtext)
+	CheckIncludedText(article, i);
     if (DoSignature)
 	article = AppendSignature(Mode == 'h', article, pwp->pw_dir, &SigLines);
     else
 	SigLines = 0;
     ProcessHeaders(AddOrg, i + SigLines, pwp);
     Length = strlen(article);
-#if	LOCAL_MAX_ARTSIZE > 0
-    if (Length > LOCAL_MAX_ARTSIZE) {
+    if ((innconf->localmaxartsize > 0) && (Length > innconf->localmaxartsize)) {
 	(void)fprintf(stderr,
 		"Article is bigger then local limit of %ld bytes\n",
-		LOCAL_MAX_ARTSIZE);
+		atoi(p));
 	QuitServer(1);
     }
-#endif	/* LOCAL_MAX_ARTSIZE > 0 */
 
     /* Do final checks. */
     if (i == 0 && HDR(_control) == NULL) {
