@@ -1,4 +1,4 @@
-/*  $Revision: 1.2 $
+/*  $Revision: 1.1.1.1 $
 **
 **  Read a C news "sys" file and split it up into a set of INN
 **  newsfeeds entries.  Also works with B news.
@@ -6,15 +6,15 @@
 **  Once done, edit all files that have HELP or all in them.
 **  Review all files, anyway.
 */
-#include "configdata.h"
 #include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include "nntp.h"
 #include "libinn.h"
-#include "clibrary.h"
 #include "macros.h"
 
 #define TEMPFILE	":tmp"
@@ -155,7 +155,6 @@ DoSub(F, p)
     char		*p;
 {
     register char	*s;
-    register BOOL	dirty;
     register int	len;
     register BOOL	SawBang;
     register BOOL	SawAll;
@@ -163,8 +162,8 @@ DoSub(F, p)
     if ((s = strtok(p, ",")) == NULL)
 	return;
 
-    dirty = FALSE;
-    len = 9;
+    (void)fprintf(F, "!*");
+    len = 8 + 1 + 2;
     do {
 	/* These are distributions, not newsgroups. */
 	if (EQ(s, "world") || EQ(s, "na") || EQ(s, "usa") || EQ(s, "inet")
@@ -178,10 +177,8 @@ DoSub(F, p)
 	    continue;
 #endif	/* defined(DO_MERGE_TO_GROUPS) */
 
-	if (dirty) {
-	    (void)putc(',', F);
-	    len++;
-	}
+	(void)putc(',', F);
+	len++;
 
 	if (len + strlen(s) + 3 > 72) {
 	    (void)fprintf(F,"\\\n\t    ");
@@ -226,7 +223,6 @@ DoSub(F, p)
 	    (void)putc('*', F);
 	    len++;
 	}
-	dirty = TRUE;
     } while ((s = strtok((char *)NULL, ",")) != NULL);
 }
 
