@@ -1,4 +1,4 @@
-/*  $Revision: 1.2 $
+/*  $Revision: 1.1.1.1 $
 **
 **  Replacement "server.h" file for remote rn.
 */
@@ -11,14 +11,19 @@
 */
 #define NNTP_NOSUCHGROUP_VAL		411
 #define NNTP_CLASS_OK			'2'
+#define NNTP_CLASS_ERROR		'4'
 #define NNTP_CLASS_FATAL		'5'
+
+#define NNTP_SYNTAX_VAL			501
 #define NNTP_STRLEN			512
 #endif	/* !defined(NNTP_CLASS_OK) */
 
 
 #define CHAR_OK			NNTP_CLASS_OK
+#define CHAR_ERR		NNTP_CLASS_ERROR
 #define CHAR_FATAL		NNTP_CLASS_FATAL
 #define ERR_NOGROUP		NNTP_NOSUCHGROUP_VAL
+#define ERR_CMDSYN		NNTP_SYNTAX_VAL
 
 
 extern char	*getserverbyfile();
@@ -27,4 +32,4 @@ extern int	server_init();
 extern void	close_server();
 extern void	put_server();
 
-extern char	ser_line[NNTP_STRLEN];
+extern char	ser_line[NNTP_STRLEN + 2];
