@@ -1,5 +1,5 @@
 /*
- * $Revision: 1.4 $
+ * $Revision: 1.6 $
  */
 
 
@@ -13,10 +13,10 @@
 **  Try to avoid changing these.
 */
 #define RELEASE "2"
-#define PATCHLEVEL "0"
-#define DATE "8-Jun-1998"
+#define PATCHLEVEL "1"
+#define DATE "24-Jul-1998"
 #define RELEASE_C 2
-#define PATCHLEVEL_C 0
+#define PATCHLEVEL_C 1
 #define SUB_PATCHLEVEL_C 0
 
 
