@@ -1,4 +1,4 @@
-/*  $Id: cc.c,v 1.40 1998/05/26 14:41:28 kondou Exp $
+/*  $Id: cc.c,v 1.42 1998/06/18 15:45:15 kondou Exp $
 **
 **  Routines for the control channel.  Create a Unix-domain datagram socket
 **  that processes on the local server send messages to.  The control
@@ -1364,7 +1364,7 @@ CCreload(av)
 	TCLreadfilter();
 #endif /* defined(DO_TCL) */
 #if defined(DO_PERL)
-        PERLreadfilter (_PATH_PERL_FILTER_INND,"filter_art") ;
+        PERLreadfilter (cpcatpath(innconf->pathfilter,_PATH_PERL_FILTER_INND),"filter_art") ;
 #endif /* defined(DO_PERL) */
 	p = "all";
     }
@@ -1432,7 +1432,7 @@ CCreload(av)
 #endif /* defined(DO_TCL) */
 #if defined(DO_PERL)
     else if (EQ(p, "filter.perl")) {
-        if (!PERLreadfilter (_PATH_PERL_FILTER_INND,"filter_art"))
+        if (!PERLreadfilter (cpcatpath(innconf->pathfilter,_PATH_PERL_FILTER_INND),"filter_art"))
             return BADPERLRELOAD ;
     }
 #endif /* defined(DO_PERL) */
@@ -2059,7 +2059,7 @@ STATIC STRING CClowmark(char *av[])
 	    break;
 	}
 	*cp++ = '\0';
-	if ((lo = atol(cp)) == 0) {
+	if ((lo = atol(cp)) == 0 && cp[0] != '0') {
 	    ret = "1 Malformed input line (missing low mark)";
 	    break;
 	}
