@@ -1,4 +1,4 @@
-/*  $Id: chan.c,v 1.25 1998/04/28 12:04:12 scrappy Exp $
+/*  $Id: chan.c,v 1.28 1998/07/02 02:16:58 kondou Exp $
 **
 **  I/O channel (and buffer) processing.
 */
@@ -144,8 +144,9 @@ CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
     cp->State = State;
     cp->Streaming = FALSE;
     cp->Skip = FALSE;
-    cp->Ihave = cp->Ihave_Duplicate = cp->Ihave_Deferred = cp->Ihave_SendIt = 0;
-    cp->Check = cp->Check_send = cp->Check_deferred = cp->Check_got = 0;
+    cp->NoResendId = FALSE;
+    cp->Ihave = cp->Ihave_Duplicate = cp->Ihave_Deferred = cp->Ihave_SendIt = cp->Ihave_Cybercan = 0;
+    cp->Check = cp->Check_send = cp->Check_deferred = cp->Check_got = cp->Check_cybercan = 0;
     cp->Takethis = cp->Takethis_Ok = cp->Takethis_Err = 0;
     cp->Size = cp->Duplicate = 0;
     cp->Unwanted_s = cp->Unwanted_f = cp->Unwanted_d = 0;
@@ -884,7 +885,7 @@ void CHANreadloop(void)
 	    if (cp->In.Size > (BIG_BUFFER)) {
 		if (cp->In.Used) {
 		    if ((cp->In.Size / cp->In.Used) > 10) {
-			cp->In.Size = (cp->In.Used * 2) < START_BUFF_SIZE ? (cp->In.Used * 2) :  START_BUFF_SIZE;
+			cp->In.Size = (cp->In.Used * 2) > START_BUFF_SIZE ? (cp->In.Used * 2) :  START_BUFF_SIZE;
 			cp->In.Data = RENEW(cp->In.Data, char, cp->In.Size);
 			cp->In.Left = cp->In.Size - cp->In.Used;
 		    }
