@@ -1,4 +1,4 @@
-/*  $Revision: 1.38 $
+/*  $Revision: 1.41 $
 **
 **  Many of the data types used here have abbreviations, such as CT
 **  for CHANNELTYPE.  Here are a list of the conventions and meanings:
@@ -138,6 +138,7 @@ typedef struct _CHANNEL {
     CHANNELSTATE	State;
     BOOL		Skip;
     BOOL		Streaming;
+    BOOL		NoResendId;
     int			fd;
     u_long		Duplicate;
     u_long		Unwanted_s;
@@ -151,6 +152,7 @@ typedef struct _CHANNEL {
     u_long		Check_send;
     u_long		Check_deferred;
     u_long		Check_got;
+    u_long		Check_cybercan;
     u_long		Takethis;
     u_long		Takethis_Ok;
     u_long		Takethis_Err;
@@ -158,6 +160,7 @@ typedef struct _CHANNEL {
     u_long		Ihave_Duplicate;
     u_long		Ihave_Deferred;
     u_long		Ihave_SendIt;
+    u_long		Ihave_Cybercan;
     int			Reported;
     long		Received;
     long		Refused;
@@ -357,7 +360,7 @@ typedef struct _WIP {
 
 typedef enum {TMR_IDLE, TMR_ARTWRITE, TMR_ARTLINK, TMR_HISWRITE,
 	      TMR_HISSYNC, TMR_SITESEND, TMR_ARTCTRL, TMR_ARTCNCL,
-	      TMR_HISHAVE, TMR_HISGREP, TMR_MAX} TMRTYPE;
+	      TMR_HISHAVE, TMR_HISGREP, TMR_PERL, TMR_MAX} TMRTYPE;
 
 
 
