@@ -1,4 +1,4 @@
-/*  $Id: nc.c,v 1.30 1998/06/03 05:56:18 kondou Exp $
+/*  $Id: nc.c,v 1.35 1998/07/16 23:04:21 kondou Exp $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
@@ -441,6 +441,10 @@ STATIC FUNCTYPE
 NCihave(CHANNEL *cp)
 {
     char	*p;
+#if defined(DO_PERL)
+    char	*perlrc;
+    int		msglen;
+#endif /* DO_PERL */
 
     cp->Ihave++;
     /* Snip off the Message-ID. */
@@ -449,6 +453,30 @@ NCihave(CHANNEL *cp)
     if (NCbadid(cp, p))
 	return;
 
+    if ((innconf->refusecybercancels) && (strncmp(p, "<cancel.", 8) == 0)) {
+	cp->Refused++;
+	cp->Ihave_Cybercan++;
+	NCwritereply(cp, NNTP_HAVEIT);
+	return;
+    }
+
+#if defined(DO_PERL)
+    /*  invoke a perl message filter on the message id */
+    if ((perlrc = (char *)HandleMessageID(p)) != NULL) {
+	cp->Refused++;
+	msglen = strlen(p) + 5; /* 3 digits + space + id + null */
+	if (cp->Sendid.Size < msglen) {
+	    if (cp->Sendid.Size > 0) DISPOSE(cp->Sendid.Data);
+	    if (msglen > MAXHEADERSIZE) cp->Sendid.Size = msglen;
+	    else cp->Sendid.Size = MAXHEADERSIZE;
+	    cp->Sendid.Data = NEW(char, cp->Sendid.Size);
+	}
+	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_GOTID_VAL, p);
+	NCwritereply(cp, cp->Sendid.Data);
+	return;
+    }
+#endif
+
     if (HIShavearticle(HashMessageID(p))) {
 	cp->Refused++;
 	cp->Ihave_Duplicate++;
@@ -928,9 +956,9 @@ STATIC FUNCTYPE NCproc(CHANNEL *cp)
 		cp->Rejected++;
 
 		/* Write a local cancel entry so nobody else gives it to us. */
-		    if (!HISremember(cp->CurrentMessageIDHash)) 
-		    	if (!HIShavearticle(cp->CurrentMessageIDHash))
-			    syslog(L_ERROR, "%s cant cancel %s", LogName, 
+		    if (!HIShavearticle(cp->CurrentMessageIDHash))
+			if (!HISremember(cp->CurrentMessageIDHash)) 
+			    syslog(L_ERROR, "%s cant write %s", LogName, 
 			        HashToText(cp->CurrentMessageIDHash)); 
 
 		/* Clear the work-in-progress entry. */
@@ -1236,6 +1264,9 @@ NCcheck(CHANNEL *cp)
 {
     char		*p;
     int			msglen;
+#if defined(DO_PERL)
+    char		*perlrc;
+#endif /* DO_PERL */
 
     cp->Check++;
     /* Snip off the Message-ID. */
@@ -1257,6 +1288,24 @@ NCcheck(CHANNEL *cp)
 	return;
     }
 
+    if ((innconf->refusecybercancels) && (strncmp(p, "<cancel.", 8) == 0)) {
+	cp->Refused++;
+	cp->Check_cybercan++;
+	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_GOTID_VAL, p);
+	NCwritereply(cp, cp->Sendid.Data);
+	return;
+    }
+
+#if defined(DO_PERL)
+    /*  invoke a perl message filter on the message id */
+    if ((perlrc = (char *)HandleMessageID(p)) != NULL) {
+	cp->Refused++;
+	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_GOTID_VAL, p);
+	NCwritereply(cp, cp->Sendid.Data);
+	return;
+    }
+#endif
+
     if (HIShavearticle(HashMessageID(p))) {
 	cp->Refused++;
 	cp->Check_got++;
@@ -1264,7 +1313,12 @@ NCcheck(CHANNEL *cp)
 	NCwritereply(cp, cp->Sendid.Data);
     } else if (WIPinprogress(p, cp, TRUE)) {
 	cp->Check_deferred++;
-	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_RESENDID_VAL, p);
+	if (cp->NoResendId) {
+	    cp->Refused++;
+	    (void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_GOTID_VAL, p);
+	} else {
+	    (void)sprintf(cp->Sendid.Data, "%d %s", NNTP_RESENDID_VAL, p);
+	}
 	NCwritereply(cp, cp->Sendid.Data);
     } else {
 	cp->Check_send++;
