@@ -1,4 +1,4 @@
-/*  $Revision: 1.34 $
+/*  $Revision: 1.41 $
 **
 */
 #include <stdio.h>
@@ -65,7 +65,7 @@ char *GetFileConfigValue(char *value)
 {
     FILE	        *F;
     int	                i;
-    char	        *p;
+    char	        *p, *q;
     char	        c;
 
     /* Read the config file. */
@@ -83,6 +83,9 @@ char *GetFileConfigValue(char *value)
 		(void)Fclose(F);
 		for (p = &ConfigBuff[i + 1]; ISWHITE(*p); p++)
 		    continue;
+		q = &p[strlen(p)-1];
+		while (q>p && ISWHITE(*q))
+		    *q-- = '\0';
 		return p;
 	    }
 	}
@@ -247,7 +250,7 @@ void SetDefaults()
     innconf->logsitename = TRUE;
     innconf->extendeddbz = FALSE;
     innconf->nnrpdoverstats = FALSE;
-    innconf->storeonxref = FALSE;
+    innconf->storeonxref = TRUE;
     innconf->decnetdomain = NULL;
     innconf->backoff_auth = FALSE;
     innconf->backoff_db = NULL;
@@ -255,6 +258,12 @@ void SetDefaults()
     innconf->backoff_postfast = 0L;
     innconf->backoff_postslow = 1L;
     innconf->backoff_trigger = 10000L;
+    innconf->refusecybercancels = FALSE;
+    innconf->nnrpdcheckart = TRUE;
+    innconf->activedenable = FALSE;
+    innconf->activedupdate = 30L;
+    innconf->activedport = 1119;
+    innconf->storemsgid = TRUE;
 }
 
 void ClearInnConf()
@@ -412,6 +421,8 @@ int ReadInnConf()
 	return(-1);
     }
     SetDefaults();
+    if (!(innconffile = getenv("INNCONF")))
+	innconffile = _PATH_CONFIG;
     syslog(L_TRACE, "Reading config from %s", innconffile); 
     /* Read the config file. */
     if ((F = Fopen(innconffile, "r", TEMPORARYOPEN)) != NULL) {
@@ -886,6 +897,36 @@ int ReadInnConf()
 		TEST_CONFIG(CONF_VAR_BACKOFFTRIGGER, bit);
 		if (!bit) innconf->backoff_trigger = atol(p);
 		SET_CONFIG(CONF_VAR_BACKOFFTRIGGER);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_REFUSECYBERCANCELS)) {
+		TEST_CONFIG(CONF_VAR_REFUSECYBERCANCELS, bit);
+		if (!bit && boolval != -1) innconf->refusecybercancels = boolval;
+		SET_CONFIG(CONF_VAR_REFUSECYBERCANCELS);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_NNRPDCHECKART)) {
+		TEST_CONFIG(CONF_VAR_NNRPDCHECKART, bit);
+		if (!bit && boolval != -1) innconf->nnrpdcheckart = boolval;
+		SET_CONFIG(CONF_VAR_NNRPDCHECKART);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_ACTIVEDENABLE)) {
+		TEST_CONFIG(CONF_VAR_ACTIVEDENABLE, bit);
+		if (!bit && boolval != -1) innconf->activedenable = boolval;
+		SET_CONFIG(CONF_VAR_ACTIVEDENABLE);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_ACTIVEDUPDATE)) {
+		TEST_CONFIG(CONF_VAR_ACTIVEDUPDATE, bit);
+		if (!bit) innconf->activedupdate = atol(p);
+		SET_CONFIG(CONF_VAR_ACTIVEDUPDATE);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_ACTIVEDPORT)) {
+		TEST_CONFIG(CONF_VAR_ACTIVEDPORT, bit);
+		if (!bit) innconf->activedport = atoi(p);
+		SET_CONFIG(CONF_VAR_ACTIVEDPORT);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_STOREMSGID)) {
+		TEST_CONFIG(CONF_VAR_STOREMSGID, bit);
+		if (!bit && boolval != -1) innconf->storemsgid = boolval;
+		SET_CONFIG(CONF_VAR_STOREMSGID);
 	    }
 	}
 	(void)Fclose(F);
