@@ -1,4 +1,4 @@
-/*  $Revision: 1.4 $
+/*  $Revision: 1.5 $
 **
 **  Library routines to let other programs control innd.
 */
@@ -59,6 +59,12 @@ ICCopen()
     int		mask;
     int		oerrno;
 
+    if (innconf == NULL) {
+	if (ReadInnConf() < 0) {
+	    ICCfailure = "innconf";
+	    return -1;
+	}
+    }
     /* Create a temporary name. */
     if (ICCsockname == NULL)
 	ICCsockname = COPY(cpcatpath(innconf->pathrun, _PATH_TEMPSOCK));
