@@ -1,29 +1,28 @@
-/*  $Revision: 1.6 $
+/*  $Revision: 1.4 $
 **
 */
 #include <stdio.h>
-#include <errno.h>
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/un.h>
 #include "configdata.h"
-#include "paths.h"
 #include "clibrary.h"
+#include <sys/socket.h>
+#include <errno.h>
+#include "paths.h"
 #include "nntp.h"
 #include "macros.h"
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
+#include <sys/un.h>
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
+#include "libinn.h"
 
 
 /*
 **  Open a connection to the local InterNetNews NNTP server and optionally
 **  create stdio FILE's for talking to it.  Return -1 on error.
 */
-int
-NNTPlocalopen(FromServerp, ToServerp, errbuff)
-    FILE		**FromServerp;
-    FILE		**ToServerp;
-    char		*errbuff;
+int NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
 {
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     int			i;
     int			j;
     int			oerrno;
@@ -42,7 +41,9 @@ NNTPlocalopen(FromServerp, ToServerp, errbuff)
     /* Connect to the server. */
     (void)memset((POINTER)&server, 0, sizeof server);
     server.sun_family = AF_UNIX;
-    (void)strcpy(server.sun_path, _PATH_NNTPCONNECT);
+    (void)strcpy(server.sun_path, innconf->pathrun);
+    (void)strcat(server.sun_path, "/");
+    (void)strcat(server.sun_path, _PATH_NNTPCONNECT);
     if (connect(i, (struct sockaddr *)&server, AF_UNIX_SOCKSIZE(server)) < 0) {
 	oerrno = errno;
 	(void)close(i);
@@ -80,10 +81,6 @@ NNTPlocalopen(FromServerp, ToServerp, errbuff)
     }
     return 0;
 #else
-    *FromServerp = NULL;
-    *ToServerp = NULL;
-    if (errbuff)
-	(void)strcpy(errbuff, "Not supported");
-    return -1;
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+    return NNTPconnect(LOOPBACK_HOST, FromServerp, ToServerp, errbuff);
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 }
