@@ -1,4 +1,4 @@
-/*  $Revision: 1.38 $
+/*  $Revision: 1.43 $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **  This server doesn't do any real load-limiting, except for what has
@@ -187,8 +187,8 @@ ExitWithStats(x)
         syslog(L_NOTICE, "%s artstats get %d time %d size %d", ClientHost,
             ARTget, ARTgettime, ARTgetsize);
     if (innconf->nnrpdoverstats && OVERcount)
-        syslog(L_NOTICE, "%s overstats count %d hit %d miss %d time %d size %d read %d dbz %d seek %d get %d", ClientHost,
-            OVERcount, OVERhit, OVERmiss, OVERtime, OVERsize, OVERread, OVERdbz, OVERseek, OVERget);
+        syslog(L_NOTICE, "%s overstats count %d hit %d miss %d time %d size %d read %d dbz %d seek %d get %d artcheck %d", ClientHost,
+            OVERcount, OVERhit, OVERmiss, OVERtime, OVERsize, OVERread, OVERdbz, OVERseek, OVERget, OVERartcheck);
 
      if (DaemonMode) {
      	shutdown(STDIN, 2);
@@ -283,6 +283,7 @@ TITLEset(what)
     union pstun un;
     
     (void)sprintf(buff, "(nnrpd) %s %s", ClientHost, what);
+    un.pst_command = buff;
     (void)pstat(PSTAT_SETCMD, un, strlen(buff), 0, 0);
 #endif	/* defined(HPUX) */
 #endif	/* defined(HAVE_SETPROCTITLE) */
@@ -695,8 +696,12 @@ main(argc, argv, env)
     int			lfd, fd;
     ARGTYPE		clen;
     struct sockaddr_in	ssa, csa;
+    struct stat		Sb;
     PID_T		pid = -1;
-   
+    GID_T               NewsGID;
+    UID_T               NewsUID;
+    int                 one = 1;
+    FILE                *pidfile;
 
 #if	!defined(HPUX)
     /* Save start and extent of argv for TITLEset. */
@@ -768,7 +773,12 @@ main(argc, argv, env)
 	    syslog(L_FATAL, "can't open socket (%m)");
 	    exit(1);
 	}
-	    
+
+	if (setsockopt(lfd, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) < 0) {
+	    syslog(L_FATAL, "can't setsockopt(SO_REUSEADDR) (%m)");
+	    exit(1);
+	}
+
 	ssa.sin_family = AF_INET;
 	ssa.sin_addr.s_addr = ListenAddr;
 	ssa.sin_port = htons(ListenPort);
@@ -779,6 +789,22 @@ main(argc, argv, env)
 	    exit(1);
 	}
 
+	/* If started as root, switch to news uid */
+	if (getuid() == 0) {
+	    if (stat(innconf->pathrun, &Sb) < 0 || !S_ISDIR(Sb.st_mode)) {
+		syslog(L_FATAL, "nnrpd cant stat %s %m", innconf->pathrun);
+		exit(1);
+	    }
+	    NewsUID = Sb.st_uid;
+	    NewsGID = Sb.st_gid;
+	    (void)setgid(NewsGID);
+	    if (getgid() != NewsGID)
+		syslog(L_ERROR, "nnrpd cant setgid to %d %m", NewsGID);
+	    (void)setuid(NewsUID);
+	    if (getuid() != NewsUID)
+		syslog(L_ERROR, "nnrpd cant setuid to %d %m", NewsUID);
+	}
+
 	/* Detach */
 	if ((pid = FORK()) < 0) {
 	    fprintf(stderr, "%s: can't ford (%s)\n", argv[0], strerror(errno));
@@ -788,7 +814,15 @@ main(argc, argv, env)
 	    exit(0);
 
 	setsid();
- 
+
+	if ((pidfile = fopen(cpcatpath(innconf->pathrun, "nnrpd.pid"),
+                                                "w")) == NULL) {
+	    syslog(L_ERROR, "cannot write nnrpd.pid %m");
+            exit(1);
+	}
+	fprintf(pidfile,"%d\n", getpid());
+	fclose(pidfile);
+
 	/* Set signal handle to care for dead children */
 	(void)signal(SIGCHLD, WaitChild);
  
@@ -811,8 +845,10 @@ listen_loop:
 	    (void)sleep(1);
 	}
 
-	if (pid != 0)
+	if (pid != 0) {
+		close(fd);
 		goto listen_loop;
+	}
 
 	/* child process starts here */
 	TITLEset("nnrpd: connected");
