@@ -14,6 +14,8 @@
 typedef off_t	CYCBUFF_OFF_T;
 /* =()<#define	CNFSseek		@<CNFSseekcall>@>()= */
 #define	CNFSseek		lseek
+/* =()<#define	CNFS_HDR_PAGESIZE	@<CNFS_HDR_PAGESIZE>@>()= */
+#define	CNFS_HDR_PAGESIZE	16384
 
 #define	CNFS_MAGICV1	"Cycbuff"	/* CNFSMASIZ bytes */
 #define	CNFS_MAGICV2	"CBuf1"		/* CNFSMASIZ bytes */
@@ -84,4 +86,11 @@ typedef struct {
   STORAGECLASS	class;		/* storage class */
 } CNFSARTHEADER;
 
+typedef struct {
+  long      zottf;      /* This should always be 0x01234*/
+  long      size;       /* Size of the article */
+  char      m_id[64];   /* We'll only store up to 63 bytes of the
+	                       Message-ID, that should be good enough */
+} oldCNFSARTHEADER;
+
 #endif	/* ! ___CNFS_PRIVATE_H */
