@@ -1,4 +1,4 @@
-/*  $Id: cnfs.c,v 1.28 1998/06/04 00:07:28 kondou Exp $
+/*  $Id: cnfs.c,v 1.39 1998/07/23 00:00:30 kondou Exp $
 **
 **  cyclic news file system
 */
@@ -338,7 +338,7 @@ STATIC BOOL CNFSparse_part_line(char *l) {
   l = ++p;
 
   /* Length/size of symbolic partition */
-  len = strtoul(l, NULL, 10) * 1024;	/* This value in KB in decimal */
+  len = strtoul(l, NULL, 10) * (CYCBUFF_OFF_T)1024;	/* This value in KB in decimal */
   if (S_ISREG(sb.st_mode) && len != sb.st_size) {
     if (sizeof(CYCBUFFEXTERN) > sb.st_size) {
       syslog(L_NOTICE, "%s: length must be at least '%ld' for '%s' cycbuff(%ld bytes)",
@@ -358,7 +358,7 @@ STATIC BOOL CNFSparse_part_line(char *l) {
   */
   minartoffset =
       cycbuff->len / (CNFS_BLOCKSIZE * 8) + CNFS_BEFOREBITF;
-  tonextblock = CNFS_BLOCKSIZE - (minartoffset & (CNFS_BLOCKSIZE - 1));
+  tonextblock = CNFS_HDR_PAGESIZE - (minartoffset & (CNFS_HDR_PAGESIZE - 1));
   cycbuff->minartoffset = minartoffset + tonextblock;
 
   if (cycbufftab == (CYCBUFF *)NULL)
@@ -816,9 +816,14 @@ STATIC int CNFSArtMayBeHere(CYCBUFF *cycbuff, CYCBUFF_OFF_T offset, U_INT32_T cy
     static int	count = 0;
     CYCBUFF	*tmp;
 
-    if (SMpreopen && (++count % 1000 == 0)) {	/* XXX 1K articles is just a guess */
-	for (tmp = cycbufftab; tmp != (CYCBUFF *)NULL; tmp = tmp->next) {
-	    CNFSReadFreeAndCycle(tmp);
+    if (SMpreopen) {
+	if (++count % 1000 == 0) {	/* XXX 1K articles is just a guess */
+	    for (tmp = cycbufftab; tmp != (CYCBUFF *)NULL; tmp = tmp->next) {
+		CNFSReadFreeAndCycle(tmp);
+	    }
+	} else if (cycnum == cycbuff->cyclenum + 1) {	/* rollover ? */
+	    CNFSReadFreeAndCycle(cycbuff);
+	    count = 0;
 	}
     }
     /*
@@ -827,7 +832,6 @@ STATIC int CNFSArtMayBeHere(CYCBUFF *cycbuff, CYCBUFF_OFF_T offset, U_INT32_T cy
     ** avoid a false negative response, *not* a false positive response.
     */
     if (! (cycnum == cycbuff->cyclenum ||
-	(cycbuff->cyclenum == 2 && cycnum + 3 == cycbuff->cyclenum) ||
 	(cycnum == cycbuff->cyclenum - 1 && offset > cycbuff->free) ||
 	(cycnum + 1 == 0 && cycbuff->cyclenum == 2 && offset > cycbuff->free))) {
 	/* We've been overwritten */
@@ -863,15 +867,22 @@ BOOL cnfs_init(BOOL *selfexpire) {
 	}
     }
     if (pagesize == 0) {
-#ifdef  XXX
+#if	defined(HAVE_GETPAGESIZE)
+	pagesize = getpagesize();
+#elif	defined(_SC_PAGESIZE)
 	if ((pagesize = sysconf(_SC_PAGESIZE)) < 0) {
-	    syslog(L_ERROR, "%s: sysconf(_SC_PAGESIZE) failed: %m", LocalLogNam);
+	    syslog(L_ERROR, "%s: sysconf(_SC_PAGESIZE) failed: %m", LocalLogName);
 	    SMseterror(SMERR_INTERNAL, "sysconf(_SC_PAGESIZE) failed");
 	    return NULL;
 	}
-#else   /* XXX */
-	pagesize = 16384;	/* XXX Need comprehensive, portable solution */
-#endif  /* XXX */
+#else
+	pagesize = 16384;
+#endif
+	if ((pagesize > CNFS_HDR_PAGESIZE) || (CNFS_HDR_PAGESIZE % pagesize)) {
+	    syslog(L_ERROR, "%s: CNFS_HDR_PAGESIZE (%d) is not a multiple of pagesize (%d)", LocalLogName, CNFS_HDR_PAGESIZE, pagesize);
+	    SMseterror(SMERR_INTERNAL, "CNFS_HDR_PAGESIZE not multiple of pagesize");
+	    return NULL;
+	}
     }
     if (STORAGE_TOKEN_LENGTH < 16) {
 	syslog(L_ERROR, "%s: token length is less than 16 bytes", LocalLogName);
@@ -1008,12 +1019,6 @@ TOKEN cnfs_store(const ARTHANDLE article, STORAGECLASS class) {
 	    (void)CNFSflushhead(metacycbuff->members[i]);
 	}
     }
-    if (metacycbuff->write_count % 1000000 == 0) {
-	syslog(L_NOTICE,
-	       "%s: cnfs_store metacycbuff %s just wrote its %ld'th article",
-	       LocalLogName, metacycbuff->name, metacycbuff->write_count);
-    }
-    /* is this stuff needed? */
     CNFSUsedBlock(cycbuff, artoffset, TRUE, TRUE);
     for (middle = artoffset + CNFS_BLOCKSIZE; middle < cycbuff->free;
 	 middle += CNFS_BLOCKSIZE) {
@@ -1100,61 +1105,122 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
         return NULL;
     }
+    if(cah.size == htonl(0x1234) && ntohl(cah.arrived) < time(NULL)-10*365*24*3600) {
+	oldCNFSARTHEADER cahh;
+	*(CNFSARTHEADER *)&cahh = cah;
+	if(read(cycbuff->fd, ((char *)&cahh)+sizeof(CNFSARTHEADER), sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER)) != sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER)) {
+            SMseterror(SMERR_UNDEFINED, "read2 failed");
+            syslog(L_ERROR, "%s: could not read2 token %s %s:0x%s:%ld: %m",
+                    LocalLogName, TokenToText(token), cycbuffname,
+                    CNFSofft2hex(offset, FALSE), cycnum);
+            DISPOSE(art);
+            if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+            return NULL;
+	}
+	cah.size = cahh.size;
+	cah.arrived = htonl(time(NULL));
+	cah.class = 0;
+	offset += sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER);
+    }
     if (cycbuff->free > cycbuff->len - CNFS_BLOCKSIZE - ntohl(cah.size) - 1) {
-        SMseterror(SMERR_UNDEFINED, "CNFSARTHEADER size overflow");
-        syslog(L_ERROR, "%s: could not match article size token %s %s:0x%s:%ld: %ld",
+        if (!SMpreopen) {
+	    SMseterror(SMERR_UNDEFINED, "CNFSARTHEADER size overflow");
+	    syslog(L_ERROR, "%s: could not match article size token %s %s:0x%s:%ld: %ld",
 		LocalLogName, TokenToText(token), cycbuffname, CNFSofft2hex(offset, FALSE), cycnum, ntohl(cah.size));
-        DISPOSE(art);
-	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
-        return NULL;
+	    DISPOSE(art);
+	    CNFSshutdowncycbuff(cycbuff);
+	    return NULL;
+	}
+	CNFSReadFreeAndCycle(cycbuff);
+	if (cycbuff->free > cycbuff->len - CNFS_BLOCKSIZE - ntohl(cah.size) - 1) {
+	    SMseterror(SMERR_UNDEFINED, "CNFSARTHEADER size overflow");
+	    syslog(L_ERROR, "%s: could not match article size token %s %s:0x%s:%ld: %ld",
+		LocalLogName, TokenToText(token), cycbuffname, CNFSofft2hex(offset, FALSE), cycnum, ntohl(cah.size));
+	    DISPOSE(art);
+	    return NULL;
+	}
     }
     private = NEW(PRIV_CNFS, 1);
     art->private = (void *)private;
     art->arrived = ntohl(cah.arrived);
-    offset += sizeof(cah);
-    pagefudge = offset % pagesize;
-    mmapoffset = offset - pagefudge;
-    private->len = pagefudge + ntohl(cah.size);
-    if ((private->base = mmap((MMAP_PTR)0, private->len, PROT_READ, MAP__ARG,
-			      cycbuff->fd, mmapoffset)) == (MMAP_PTR) -1) {
-        SMseterror(SMERR_UNDEFINED, "mmap failed");
-        syslog(L_ERROR, "%s: could not mmap token %s %s:0x%s:%ld: %m",
-	        LocalLogName, TokenToText(token), cycbuffname, CNFSofft2hex(offset, FALSE), cycnum);
-        DISPOSE(art->private);
-        DISPOSE(art);
-	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
-        return NULL;
+    if (innconf->articlemmap) {
+	offset += sizeof(cah);
+	pagefudge = offset % pagesize;
+	mmapoffset = offset - pagefudge;
+	private->len = pagefudge + ntohl(cah.size);
+	if ((private->base = mmap((MMAP_PTR)0, private->len, PROT_READ,
+		MAP__ARG, cycbuff->fd, mmapoffset)) == (MMAP_PTR) -1) {
+	    SMseterror(SMERR_UNDEFINED, "mmap failed");
+	    syslog(L_ERROR, "%s: could not mmap token %s %s:0x%s:%ld: %m",
+		LocalLogName, TokenToText(token), cycbuffname, CNFSofft2hex(offset, FALSE), cycnum);
+	    DISPOSE(art->private);
+	    DISPOSE(art);
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return NULL;
+	}
+    } else {
+	private->base = NEW(char, ntohl(cah.size));
+	if (read(cycbuff->fd, private->base, ntohl(cah.size)) < 0) {
+	    SMseterror(SMERR_UNDEFINED, "read failed");
+	    syslog(L_ERROR, "%s: could not read token %s %s:0x%s:%ld: %m",
+		LocalLogName, TokenToText(token), cycbuffname, CNFSofft2hex(offset, FALSE), cycnum);
+	    DISPOSE(private->base);
+	    DISPOSE(art->private);
+	    DISPOSE(art);
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return NULL;
+	}
     }
     ret_token = token;    
     art->token = &ret_token;
     art->len = ntohl(cah.size);
     if (amount == RETR_ALL) {
-	art->data = private->base + pagefudge;
+	art->data = innconf->articlemmap ? private->base + pagefudge : private->base;
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return art;
     }
-    if ((p = SMFindBody(private->base + pagefudge, art->len)) == NULL) {
+    if ((p = SMFindBody(innconf->articlemmap ? private->base + pagefudge : private->base, art->len)) == NULL) {
         SMseterror(SMERR_NOBODY, NULL);
-	munmap(private->base, private->len);
+	if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && !defined(_nec_ews)
+	    madvise(private->base, private->len, MADV_DONTNEED);
+#endif
+	    munmap(private->base, private->len);
+	} else
+	    DISPOSE(private->base);
         DISPOSE(art->private);
         DISPOSE(art);
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
         return NULL;
     }
     if (amount == RETR_HEAD) {
-	art->data = private->base + pagefudge;
-        art->len = p - private->base - pagefudge;
+	if (innconf->articlemmap) {
+	    art->data = private->base + pagefudge;
+            art->len = p - private->base - pagefudge;
+	} else {
+	    art->data = private->base;
+            art->len = p - private->base;
+	}
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
         return art;
     }
     if (amount == RETR_BODY) {
         art->data = p;
-	art->len = art->len - (p - private->base - pagefudge);
+	if (innconf->articlemmap)
+	    art->len = art->len - (p - private->base - pagefudge);
+	else
+	    art->len = art->len - (p - private->base);
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
         return art;
     }
     SMseterror(SMERR_UNDEFINED, "Invalid retrieve request");
-    munmap(private->base, private->len);
+    if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && !defined(_nec_ews)
+	madvise(private->base, private->len, MADV_DONTNEED);
+#endif
+	munmap(private->base, private->len);
+    } else
+	DISPOSE(private->base);
     DISPOSE(art->private);
     DISPOSE(art);
     if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
@@ -1169,10 +1235,13 @@ void cnfs_freearticle(ARTHANDLE *article) {
     
     if (article->private) {
 	private = (PRIV_CNFS *)article->private;
+	if (innconf->articlemmap) {
 #if defined(MADV_DONTNEED) && !defined(_nec_ews)
-	madvise(private->base, private->len, MADV_DONTNEED);
+	    madvise(private->base, private->len, MADV_DONTNEED);
 #endif	/* MADV_DONTNEED */
-	munmap(private->base, private->len);
+	    munmap(private->base, private->len);
+	} else
+	    DISPOSE(private->base);
 	DISPOSE(private);
     }
     DISPOSE(article);
@@ -1203,7 +1272,6 @@ BOOL cnfs_cancel(TOKEN token) {
 	return FALSE;
     }
     if (! (cycnum == cycbuff->cyclenum ||
-	(cycbuff->cyclenum == 2 && cycnum + 3 == cycbuff->cyclenum) ||
 	(cycnum == cycbuff->cyclenum - 1 && offset > cycbuff->free) ||
 	(cycnum + 1 == 0 && cycbuff->cyclenum == 2 && offset > cycbuff->free))) {
 	SMseterror(SMERR_NOENT, NULL);
@@ -1242,10 +1310,13 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	priv = *(PRIV_CNFS *)article->private;
 	DISPOSE(article->private);
 	DISPOSE(article);
+	if (innconf->articlemmap) {
 #if defined(MADV_DONTNEED) && !defined(_nec_ews)
-	madvise(priv.base, priv.len, MADV_DONTNEED);  
+	    madvise(priv.base, priv.len, MADV_DONTNEED);  
 #endif
-	munmap(priv.base, priv.len);
+	    munmap(priv.base, priv.len);
+	} else
+	    DISPOSE(priv.base);
 	cycbuff = priv.cycbuff;
     }
 
@@ -1303,6 +1374,19 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return (ARTHANDLE *)NULL;
     }
+    if(cah.size == htonl(0x1234) && ntohl(cah.arrived) < time(NULL)-10*365*24*3600) {
+	oldCNFSARTHEADER cahh;
+	*(CNFSARTHEADER *)&cahh = cah;
+	if(read(cycbuff->fd, ((char *)&cahh)+sizeof(CNFSARTHEADER), sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER)) != sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER)) {
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return (ARTHANDLE *)NULL;
+	}
+	cah.size = cahh.size;
+	cah.arrived = htonl(time(NULL));
+	cah.class = 0;
+	priv.offset += sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER);
+	offset += sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER);
+    }
     art = NEW(ARTHANDLE, 1);
     private = NEW(PRIV_CNFS, 1);
     art->private = (void *)private;
@@ -1322,38 +1406,57 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
     art->arrived = ntohl(cah.arrived);
     token = CNFSMakeToken(cycbuff->name, offset, cycbuff->cyclenum, ntohl(cah.class), (TOKEN *)NULL);
     art->token = &token;
-    offset += sizeof(cah);
-    pagefudge = offset % pagesize;
-    mmapoffset = offset - pagefudge;
-    private->len = pagefudge + ntohl(cah.size);
-    if ((private->base = mmap((MMAP_PTR)0, private->len, PROT_READ, MAP__ARG,
-	cycbuff->fd, mmapoffset)) == (MMAP_PTR) -1) {
-	art->data = NULL;
-	art->len = 0;
-	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
-	return art;
+    if (innconf->articlemmap) {
+	offset += sizeof(cah);
+	pagefudge = offset % pagesize;
+	mmapoffset = offset - pagefudge;
+	private->len = pagefudge + ntohl(cah.size);
+	if ((private->base = mmap((MMAP_PTR)0, private->len, PROT_READ,
+	    MAP__ARG, cycbuff->fd, mmapoffset)) == (MMAP_PTR) -1) {
+	    art->data = NULL;
+	    art->len = 0;
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return art;
+	}
+    } else {
+	private->base = NEW(char, ntohl(cah.size));
+	if (read(cycbuff->fd, private->base, ntohl(cah.size)) < 0) {
+	    art->data = NULL;
+	    art->len = 0;
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return art;
+	    DISPOSE(private->base);
+	}
     }
     art->len = ntohl(cah.size);
     if (amount == RETR_ALL) {
-	art->data = private->base + pagefudge;
+	art->data = innconf->articlemmap ? private->base + pagefudge : private->base;
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return art;
     }
-    if ((p = SMFindBody(private->base + pagefudge, art->len)) == NULL) {
+    if ((p = SMFindBody(innconf->articlemmap ? private->base + pagefudge : private->base, art->len)) == NULL) {
 	art->data = NULL;
 	art->len = 0;
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return art;
     }
     if (amount == RETR_HEAD) {
-	art->data = private->base + pagefudge;
-	art->len = p - private->base - pagefudge;
+	if (innconf->articlemmap) {
+	    art->data = private->base + pagefudge;
+	    art->len = p - private->base - pagefudge;
+	} else {
+	    art->data = private->base;
+	    art->len = p - private->base;
+	}
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return art;
     }
     if (amount == RETR_BODY) {
 	art->data = p;
-	art->len = art->len - (p - private->base - pagefudge);
+	if (innconf->articlemmap)
+	    art->len = art->len - (p - private->base - pagefudge);
+	else
+	    art->len = art->len - (p - private->base);
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return art;
     }
